# Generated by roxygen2: do not edit by hand

export(corrgram)
export(corrgram2)
export(grid_panel.conf)
export(grid_panel.ellipse)
export(grid_panel.fill)
export(grid_panel.pie)
export(grid_panel.pts)
export(grid_panel.shade)
export(grid_text.panel)
export(levelplot_panel.ellipse)
export(levelplot_panel.pie)
export(panel.bar)
export(panel.conf)
export(panel.cor)
export(panel.density)
export(panel.ellipse)
export(panel.fill)
export(panel.minmax)
export(panel.pie)
export(panel.pts)
export(panel.shade)
export(panel.txt)
export(splom_panel.ellipse)
export(splom_panel.pie)
export(splom_panel.shade)
import(grDevices)
import(graphics)
import(grid)
import(stats)
importFrom(ellipse,ellipse)
importFrom(lattice,current.panel.limits)
importFrom(lattice,level.colors)
importFrom(lattice,levelplot)
importFrom(lattice,llines)
importFrom(lattice,panel.polygon)
importFrom(lattice,panel.rect)
importFrom(lattice,panel.text)
importFrom(lattice,panel.xyplot)
importFrom(lattice,splom)
importFrom(utils,installed.packages)
