## ----setup, include=FALSE---------------------------------------------------------------
library(corrgram)

## ----vote-example, fig.width=6, fig.height=6--------------------------------------------
corrgram2(vote, abs = TRUE, order = TRUE, legend = TRUE, title = "vote data")

## ----auto-example, fig.width=6, fig.height=6--------------------------------------------
vars6 <- setdiff(colnames(auto), c("Model", "Origin"))
corrgram2(auto[, vars6], 
          lower.panel = grid_panel.shade, upper.panel=grid_panel.pie,
          title = "auto data", legend = TRUE)

