% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corrgram_levelplot.R
\name{levelplot_panel.ellipse}
\alias{levelplot_panel.ellipse}
\alias{levelplot_panel.pie}
\title{Panel functions for lattice corrgrams via levelplot}
\usage{
levelplot_panel.ellipse(
  x,
  y,
  z,
  subscripts,
  at,
  level = 0.9,
  label = FALSE,
  ...
)

levelplot_panel.pie(x, y, z, subscripts, at = pretty(z), scale = 0.9, ...)
}
\arguments{
\item{x}{Numeric coordinates from levelplot.}

\item{y}{Numeric coordinates from levelplot.}

\item{z}{Correlation values from levelplot.}

\item{subscripts}{Subscripts for lattice panel. (not used)}

\item{at}{Breaks for color levels.}

\item{level}{Confidence level for ellipse (default 0.9).}

\item{label}{Logical; if TRUE, show correlation values as text.}

\item{...}{Additional arguments passed to panel functions.}

\item{scale}{Numeric; scaling factor for pie size (default 0.8).}
}
\description{
These functions provide custom panel methods for \code{lattice::levelplot()}.
}
\details{
\code{levelplot_panel.ellipse} Draws ellipses representing correlation coefficients
in the upper triangle of a matrix. Optionally adds numeric labels in the 
lower triangle.

\code{levelplot_panel.pie} Draws pie glyphs representing correlation coefficients,
omitting the diagonal.
}
\examples{
library(lattice)
levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
            xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
            scales = list(x = list(rot = 90)), 
            panel = levelplot_panel.ellipse, label = TRUE)
levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
            xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
            scales = list(x = list(rot = 90)), 
            panel = levelplot_panel.pie, label = TRUE)
}
